/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2014 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

/**
 * This file handles the geoid separation
 * @file Geoid.cpp
 * @see http://en.wikipedia.org/wiki/EGM96
 */

#include "Geoid.hpp"
#include "Geo/GeoPoint.hpp"

#include <stdint.h>

#define EGM96SIZE 16200

extern const uint8_t egm96data[] asm("_binary_egm96s_dem_start");

fixed
EGM96::LookupSeparation(const GeoPoint &pt)
{
  int ilat, ilon;
  ilat = iround((Angle::QuarterCircle() - pt.latitude).Half().Degrees());
  ilon = iround(pt.longitude.AsBearing().Half().Degrees());

  int offset = ilat * 180 + ilon;
  if (offset >= EGM96SIZE)
    return fixed(0);
  if (offset < 0)
    return fixed(0);

  return fixed((int)egm96data[offset] - 127);
}
